#include <asf.h>
#include "string.h"
#include "my_variable.h"

#define debug							UART2
#define CONF_UART_BAUDRATE				256000
#define CONF_UART_CHAR_LENGTH			US_MR_CHRL_8_BIT
#define CONF_UART_PARITY				US_MR_PAR_NO
#define CONF_UART_STOP_BITS				US_MR_NBSTOP_1_BIT


#define tx1								PIO_PB1_IDX
#define rx1								PIO_PB0_IDX
#define clk1							PIO_PB13_IDX

#define tx2								PIO_PD28_IDX
#define rx2								PIO_PD30_IDX
#define clk2							PIO_PA0_IDX

#define tx3								PIO_PB2_IDX
#define rx3								PIO_PB3_IDX
#define clk3							PIO_PC19_IDX



/**
 * \This function gives clock pulses to Sensor 1 on CLOCK pin
 * INPUT: a : Number of pulses required 
 */
void  pulse_sclk_1(int a)
{
	while(a--)
	{
		ioport_set_pin_level(clk1,1);for(int x=0;x<30;x++);
		ioport_set_pin_level(clk1,0);for(int x=0;x<30;x++);
	}
}
/**
 * \This function gives clock pulses to Sensor 2 on CLOCK pin
 * INPUT: a : Number of pulses required 
 */
void  pulse_sclk_2(int a)
{
	while(a--)
	{
		ioport_set_pin_level(clk2,1);for(int x=0;x<30;x++);
		ioport_set_pin_level(clk2,0);for(int x=0;x<30;x++);
	}
}
/**
 * \This function gives clock pulses to Sensor 3 on CLOCK pin
 * INPUT: a : Number of pulses required 
 */
void  pulse_sclk_3(int a)
{
	while(a--)
	{
		ioport_set_pin_level(clk3,1);for(int x=0;x<30;x++);
		ioport_set_pin_level(clk3,0);for(int x=0;x<30;x++);
	}
}
/**
 * \This function Sends data(1 byte) to sensor 1 over TX pin
 * INPUT: dat : data byte 
 */
void send_1(unsigned char dat)
{
	int i;
	ioport_set_pin_level(tx1,0);
	pulse_sclk_1(1);
	for(i=0;i<8;i++)
	{
		if( (dat&0x01)==0x01)
		ioport_set_pin_level(tx1,1);
		else
		ioport_set_pin_level(tx1,0);
		pulse_sclk_1(1) ;
		dat>>=1;
	}
	ioport_set_pin_level(tx1,1);
	pulse_sclk_1(1);
}
/**
 * \This function Sends data(1 byte) to sensor 2 over TX pin
 * INPUT: dat : data byte 
 */
void send_2(unsigned char dat)
{
	int i;
	ioport_set_pin_level(tx2,0);
	pulse_sclk_2(1);
	for(i=0;i<8;i++)
	{
		if( (dat&0x01)==0x01)
		ioport_set_pin_level(tx2,1);
		else
		ioport_set_pin_level(tx2,0);
		pulse_sclk_2(1) ;
		dat>>=1;
	}
	ioport_set_pin_level(tx2,1);
	pulse_sclk_2(1);
}
/**
 * \This function Sends data(1 byte) to sensor 3 over TX pin
 * INPUT: dat : data byte 
 */
void send_3(unsigned char dat)
{
	int i;
	ioport_set_pin_level(tx3,0);
	pulse_sclk_3(1);
	for(i=0;i<8;i++)
	{
		if( (dat&0x01)==0x01)
		ioport_set_pin_level(tx3,1);
		else
		ioport_set_pin_level(tx3,0);
		pulse_sclk_3(1) ;
		dat>>=1;
	}
	ioport_set_pin_level(tx3,1);
	pulse_sclk_3(1);
}
/**
 * \This function receives a byte from sensor 1 over RX pin
 * RETURN: unsigned char : Read byte
 */
char receive_1()
{
	int i;
	char dat=0;
	pulse_sclk_1(1)   ;
	for(i=0;i<8;i++)
	{
		dat>>=1;
		if(ioport_get_pin_level(rx1)==1)
		dat|=0x80;
		pulse_sclk_1(1) ;
	}
	pulse_sclk_1(1) ;
	return (dat);
}
/**
 * \This function receives a byte from sensor 2 over RX pin
 * RETURN: unsigned char : Read byte
 */
char receive_2()
{
	int i;
	char dat=0;
	pulse_sclk_2(1)   ;
	for(i=0;i<8;i++)
	{
		dat>>=1;
		if(ioport_get_pin_level(rx2)==1)
		dat|=0x80;
		pulse_sclk_2(1) ;
	}
	pulse_sclk_2(1) ;
	return (dat);
	
}
/**
 * \This function receives a byte from sensor 3 over RX pin
 * RETURN: unsigned char : Read byte
 */
char receive_3()
{
	int i;
	char dat;	dat=0;
	pulse_sclk_3(1)   ;
	for(i=0;i<8;i++)
	{
		dat>>=1;
		if(ioport_get_pin_level(rx3)==1)
		dat|=0x80;
		pulse_sclk_3(1) ;
	}
	pulse_sclk_3(1) ;
	return (dat);
	
}
/**
 * \This function Resets all sensors
 */
void reset()
{
	ioport_set_pin_level(clk1,0);
	ioport_set_pin_level(tx1,0);
	pulse_sclk_1(30);
	ioport_set_pin_level(tx1,1);
	pulse_sclk_1(10);
	send_1(0x1B);
	pulse_sclk_1(5);
	send_1(0x5F);
	send_1(0x00);
	
	ioport_set_pin_level(clk2,0);
	ioport_set_pin_level(tx2,0);
	pulse_sclk_2(30);
	ioport_set_pin_level(tx2,1);
	pulse_sclk_2(10);
	send_2(0x1B);
	pulse_sclk_2(5);
	send_2(0x5F);
	send_2(0x00);
	
	ioport_set_pin_level(clk3,0);
	ioport_set_pin_level(tx3,0);
	pulse_sclk_3(30);
	ioport_set_pin_level(tx3,1);
	pulse_sclk_3(10);
	send_3(0x1B);
	pulse_sclk_3(5);
	send_3(0x5F);
	send_3(0x00);
	
}
/**
 * \This function reads 102 pixels and stores in a buffer
 * INPUT: a - if 1 reads 102 pixels from sensor 1
 *			  If 2 reads 102 pixels from sensor 2
 *			  If 3 reads 102 pixels from sensor 3
 */
void read_pixel(int a)
{
	static int i;
	if(a==1)
	{
		for(i=0;i<102;i++)
			sens_data_1[i] = receive_1();
	}
	else if(a==2)
	{
		for(i=0;i<102;i++)
			sens_data_2[i] = receive_2();
	}
	else if(a==3)
	{
		for(i=0;i<102;i++)
			sens_data_3[i] = receive_3();
	}
}
/**
 * \This function sets offsets and gain for all sensor
 * INPUT: offset : offset value
 *        GC   : gain value
 */
void set_offset_gain(char offset,char GC)
{
	//if(offset<0)offset*=-1;
	send_1(0x40)   ;
	send_1(offset)  ;
	send_1(0x41);
	send_1(GC);
	send_1(0x42)   ;
	send_1(offset)  ;
	send_1(0x43)  ;
	send_1(GC)   ;
	send_1(0x44)  ;
	send_1(offset)   ;
	send_1(0x45);
	send_1(GC) ;
	
	send_2(0x40)   ;
	send_2(offset)  ;
	send_2(0x41);
	send_2(GC);
	send_2(0x42)   ;
	send_2(offset)  ;
	send_2(0x43)  ;
	send_2(GC)   ;
	send_2(0x44)  ;
	send_2(offset)   ;
	send_2(0x45);
	send_2(GC) ;
	
	send_3(0x40)   ;
	send_3(offset)  ;
	send_3(0x41);
	send_3(GC);
	send_3(0x42)   ;
	send_3(offset)  ;
	send_3(0x43)  ;
	send_3(GC)   ;
	send_3(0x44)  ;
	send_3(offset)   ;
	send_3(0x45);
	send_3(GC) ;
}

/**
 * \This function sends start Integration command to sensor 1
 */
void start_int_1()
{
	send_1(0x08);pulse_sclk_1(22)  ;
}
/**
 * \This function sends start Integration command to sensor 2
 */
void start_int_2()
{
	send_2(0x08);pulse_sclk_2(22)  ;
}
/**
 * \This function sends start Integration command to sensor 3
 */
void start_int_3()
{
	send_3(0x08);pulse_sclk_3(22)  ;
}
/**
 * \This function sends stop Integration command to sensor 1
 */
void stop_int_1()
{
	send_1(0x10);pulse_sclk_1(5);
}
/**
 * \This function sends stop Integration command to sensor 2
 */
void stop_int_2()
{
	send_2(0x10);pulse_sclk_2(5);
}
/**
 * \This function sends stop Integration command to sensor 3
 */
void stop_int_3()
{
	send_3(0x10);pulse_sclk_3(5);
}
/**
 * \This function sends start command to sensor 1 and wait for ACK
 */
void start_read_1()
{
	send_1(0x02);	pulse_sclk_1(1);
	while(ioport_get_pin_level(rx1)==1)	pulse_sclk_1(1);
}
/**
 * \This function sends start command to sensor 2 and wait for ACK
 */
void start_read_2()
{
	send_2(0x02);	pulse_sclk_2(1);
	while(ioport_get_pin_level(rx2)==1)	pulse_sclk_2(1);
}
/**
 * \This function sends start command to sensor 3 and wait for ACK
 */
void start_read_3()
{
	send_3(0x02);	pulse_sclk_3(1);
	while(ioport_get_pin_level(rx3)==1)	pulse_sclk_3(1);
}

/**
 * \Initialization of UART, PWM, GPIOS 
 */
void initialization()
{
	//Initialize Serial Debug UART
	static usart_serial_options_t usart_options = {
		.baudrate =   CONF_UART_BAUDRATE,
		.charlength = CONF_UART_CHAR_LENGTH,
		.paritytype = CONF_UART_PARITY,
		.stopbits =   CONF_UART_STOP_BITS
	};
	WDT->WDT_MR = WDT_MR_WDDIS;     // disable watchdog
	ioport_init();                  // call before using IOPORT service
	// configure UART pins
	ioport_set_port_mode(IOPORT_PIOD, PIO_PD25C_URXD2 | PIO_PD26C_UTXD2, IOPORT_MODE_MUX_C);
	ioport_disable_port(IOPORT_PIOD, PIO_PD25C_URXD2 | PIO_PD26C_UTXD2);
	// initialize the UART
	usart_serial_init(debug, &usart_options);
	// send a string using the write packet function
	
	
	//Initialize Sensor Communication pins
	
	ioport_set_pin_dir(tx1,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(clk1,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(rx1,IOPORT_DIR_INPUT);
	
	ioport_set_pin_dir(tx2,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(clk2,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(rx2,IOPORT_DIR_INPUT);
	
	ioport_set_pin_dir(tx3,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(clk3,IOPORT_DIR_OUTPUT);
	ioport_set_pin_dir(rx3,IOPORT_DIR_INPUT);
	

}


/**
 * \Main function 
 */
int main (void)
{
	
	pwm_channel_t g_pwm_channel_led;
	sysclk_init(); // system clock initialization
	initialization();// Peripheral Initialization
	reset();		// Reset all sensors
	set_offset_gain(0,0);//set offset/gain for all sensor

	delay_ms(2);
	
	while(1)
	{
		start_int_1();
		delay_us(250);
		stop_int_1();
		start_read_1();
		read_pixel(1);
	
		usart_serial_write_packet(debug, "\n\r1=",4);
		for(int i=0;i<102;i++)
		{
			sprintf(buff,"%d ",sens_data_1[i]);
			usart_serial_write_packet(debug, buff,strlen(buff));
		}
		
		
	}
	
	
	
	
	

}

